@@ -1,4 +1,4 @@
-/* $OpenBSD: e_chacha.c,v 1.5 2014/08/04 04:16:11 miod Exp $ */
+/* $OpenBSD: e_chacha.c,v 1.8 2020/01/26 07:47:26 tb Exp $ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  *
@@ -34,8 +34,17 @@ static const EVP_CIPHER chacha20_cipher = {
 	.nid = NID_chacha20,
 	.block_size = 1,
 	.key_len = 32,
-	.iv_len = 8,
-	.flags = EVP_CIPH_STREAM_CIPHER,
+	/* 
+	 * The 128 bit EVP IV is split for ChaCha into four 32 bit pieces:
+	 * 			counter[0]	counter[1]	iv[0]	iv[1]
+	 * OpenSSL exposes these as:
+	 * 	openssl_iv =	counter[0]	iv[0]		iv[1]	iv[2]
+	 * Due to the cipher internal state's symmetry, these are functionally
+	 * equivalent.
+	 */
+	.iv_len = 16,
+	.flags = EVP_CIPH_STREAM_CIPHER | EVP_CIPH_ALWAYS_CALL_INIT |
+	    EVP_CIPH_CUSTOM_IV,
 	.init = chacha_init,
 	.do_cipher = chacha_cipher,
 	.ctx_size = sizeof(ChaCha_ctx)
@@ -49,12 +58,17 @@ EVP_chacha20(void)
 
 static int
 chacha_init(EVP_CIPHER_CTX *ctx, const unsigned char *key,
-    const unsigned char *iv, int enc)
+    const unsigned char *openssl_iv, int enc)
 {
-	ChaCha_set_key((ChaCha_ctx *)ctx->cipher_data, key,
-	    EVP_CIPHER_CTX_key_length(ctx) * 8);
-	if (iv != NULL)
-		ChaCha_set_iv((ChaCha_ctx *)ctx->cipher_data, iv, NULL);
+	if (key != NULL)
+		ChaCha_set_key((ChaCha_ctx *)ctx->cipher_data, key,
+		    EVP_CIPHER_CTX_key_length(ctx) * 8);
+	if (openssl_iv != NULL) {
+		const unsigned char *iv = openssl_iv + 8;
+		const unsigned char *counter = openssl_iv;
+
+		ChaCha_set_iv((ChaCha_ctx *)ctx->cipher_data, iv, counter);
+	}
 	return 1;
 }
 
