@@ -1,4 +1,4 @@
-/* $OpenBSD: s3_lib.c,v 1.187 2019/10/04 17:21:24 jsing Exp $ */
+/* $OpenBSD: s3_lib.c,v 1.191 2020/02/16 14:33:04 inoguchi Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -1563,13 +1563,10 @@ ssl3_free(SSL *s)
 
 	DH_free(S3I(s)->tmp.dh);
 	EC_KEY_free(S3I(s)->tmp.ecdh);
-
 	freezero(S3I(s)->tmp.x25519, X25519_KEY_LENGTH);
 
+	tls13_key_share_free(S3I(s)->hs_tls13.key_share);
 	tls13_secrets_destroy(S3I(s)->hs_tls13.secrets);
-	freezero(S3I(s)->hs_tls13.x25519_private, X25519_KEY_LENGTH);
-	freezero(S3I(s)->hs_tls13.x25519_public, X25519_KEY_LENGTH);
-	freezero(S3I(s)->hs_tls13.x25519_peer_public, X25519_KEY_LENGTH);
 	freezero(S3I(s)->hs_tls13.cookie, S3I(s)->hs_tls13.cookie_len);
 
 	sk_X509_NAME_pop_free(S3I(s)->tmp.ca_names, X509_NAME_free);
@@ -1599,21 +1596,19 @@ ssl3_clear(SSL *s)
 	S3I(s)->tmp.dh = NULL;
 	EC_KEY_free(S3I(s)->tmp.ecdh);
 	S3I(s)->tmp.ecdh = NULL;
+	S3I(s)->tmp.ecdh_nid = NID_undef;
+	freezero(S3I(s)->tmp.x25519, X25519_KEY_LENGTH);
+	S3I(s)->tmp.x25519 = NULL;
+
 	freezero(S3I(s)->hs.sigalgs, S3I(s)->hs.sigalgs_len);
 	S3I(s)->hs.sigalgs = NULL;
 	S3I(s)->hs.sigalgs_len = 0;
 
-	freezero(S3I(s)->tmp.x25519, X25519_KEY_LENGTH);
-	S3I(s)->tmp.x25519 = NULL;
+	tls13_key_share_free(S3I(s)->hs_tls13.key_share);
+	S3I(s)->hs_tls13.key_share = NULL;
 
 	tls13_secrets_destroy(S3I(s)->hs_tls13.secrets);
 	S3I(s)->hs_tls13.secrets = NULL;
-	freezero(S3I(s)->hs_tls13.x25519_private, X25519_KEY_LENGTH);
-	S3I(s)->hs_tls13.x25519_private = NULL;
-	freezero(S3I(s)->hs_tls13.x25519_public, X25519_KEY_LENGTH);
-	S3I(s)->hs_tls13.x25519_public = NULL;
-	freezero(S3I(s)->hs_tls13.x25519_peer_public, X25519_KEY_LENGTH);
-	S3I(s)->hs_tls13.x25519_peer_public = NULL;
 	freezero(S3I(s)->hs_tls13.cookie, S3I(s)->hs_tls13.cookie_len);
 	S3I(s)->hs_tls13.cookie = NULL;
 	S3I(s)->hs_tls13.cookie_len = 0;
@@ -2240,6 +2235,16 @@ _SSL_CTX_add_extra_chain_cert(SSL_CTX *ctx, X509 *cert)
 
 static int
 _SSL_CTX_get_extra_chain_certs(SSL_CTX *ctx, STACK_OF(X509) **certs)
+{
+	*certs = ctx->extra_certs;
+	if (*certs == NULL)
+		*certs = ctx->internal->cert->key->chain;
+
+	return 1;
+}
+
+static int
+_SSL_CTX_get_extra_chain_certs_only(SSL_CTX *ctx, STACK_OF(X509) **certs)
 {
 	*certs = ctx->extra_certs;
 	return 1;
@@ -2325,7 +2330,10 @@ ssl3_ctx_ctrl(SSL_CTX *ctx, int cmd, long larg, void *parg)
 		return _SSL_CTX_add_extra_chain_cert(ctx, parg);
 
 	case SSL_CTRL_GET_EXTRA_CHAIN_CERTS:
-		return _SSL_CTX_get_extra_chain_certs(ctx, parg);
+		if (larg == 0)
+			return _SSL_CTX_get_extra_chain_certs(ctx, parg);
+		else
+			return _SSL_CTX_get_extra_chain_certs_only(ctx, parg);
 
 	case SSL_CTRL_CLEAR_EXTRA_CHAIN_CERTS:
 		return _SSL_CTX_clear_extra_chain_certs(ctx);
@@ -2489,6 +2497,16 @@ ssl3_choose_cipher(SSL *s, STACK_OF(SSL_CIPHER) *clnt,
 		    !SSL_USE_TLS1_2_CIPHERS(s))
 			continue;
 
+		/* Skip TLS v1.3 only ciphersuites if not supported. */
+		if ((c->algorithm_ssl & SSL_TLSV1_3) &&
+		    !SSL_USE_TLS1_3_CIPHERS(s))
+			continue;
+
+		/* If TLS v1.3, only allow TLS v1.3 ciphersuites. */
+		if (SSL_USE_TLS1_3_CIPHERS(s) &&
+		    !(c->algorithm_ssl & SSL_TLSV1_3))
+			continue;
+
 		ssl_set_cert_masks(cert, c);
 		mask_k = cert->mask_k;
 		mask_a = cert->mask_a;
@@ -2496,7 +2514,6 @@ ssl3_choose_cipher(SSL *s, STACK_OF(SSL_CIPHER) *clnt,
 		alg_k = c->algorithm_mkey;
 		alg_a = c->algorithm_auth;
 
-
 		ok = (alg_k & mask_k) && (alg_a & mask_a);
 
 		/*
