@@ -1,6 +1,6 @@
-/* $OpenBSD: ssl_sigalgs.c,v 1.11 2018/11/16 02:41:16 beck Exp $ */
+/* $OpenBSD: ssl_sigalgs.c,v 1.20 2019/04/01 02:09:21 beck Exp $ */
 /*
- * Copyright (c) 2018, Bob Beck <beck@openbsd.org>
+ * Copyright (c) 2018-2019 Bob Beck <beck@openbsd.org>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -29,13 +29,11 @@ const struct ssl_sigalg sigalgs[] = {
 		.value = SIGALG_RSA_PKCS1_SHA512,
 		.md = EVP_sha512,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 	},
 	{
-		.value = SIGALG_ECDSA_SECP512R1_SHA512,
+		.value = SIGALG_ECDSA_SECP521R1_SHA512,
 		.md = EVP_sha512,
 		.key_type = EVP_PKEY_EC,
-		.pkey_idx = SSL_PKEY_ECC,
 		.curve_nid = NID_secp521r1,
 	},
 #ifndef OPENSSL_NO_GOST
@@ -43,33 +41,28 @@ const struct ssl_sigalg sigalgs[] = {
 		.value = SIGALG_GOSTR12_512_STREEBOG_512,
 		.md = EVP_streebog512,
 		.key_type = EVP_PKEY_GOSTR12_512,
-		.pkey_idx = SSL_PKEY_GOST01, /* XXX */
 	},
 #endif
 	{
 		.value = SIGALG_RSA_PKCS1_SHA384,
 		.md = EVP_sha384,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 	},
 	{
 		.value = SIGALG_ECDSA_SECP384R1_SHA384,
 		.md = EVP_sha384,
 		.key_type = EVP_PKEY_EC,
-		.pkey_idx = SSL_PKEY_ECC,
 		.curve_nid = NID_secp384r1,
 	},
 	{
 		.value = SIGALG_RSA_PKCS1_SHA256,
 		.md = EVP_sha256,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 	},
 	{
 		.value = SIGALG_ECDSA_SECP256R1_SHA256,
 		.md = EVP_sha256,
 		.key_type = EVP_PKEY_EC,
-		.pkey_idx = SSL_PKEY_ECC,
 		.curve_nid = NID_X9_62_prime256v1,
 	},
 #ifndef OPENSSL_NO_GOST
@@ -77,85 +70,72 @@ const struct ssl_sigalg sigalgs[] = {
 		.value = SIGALG_GOSTR12_256_STREEBOG_256,
 		.md = EVP_streebog256,
 		.key_type = EVP_PKEY_GOSTR12_256,
-		.pkey_idx = SSL_PKEY_GOST01, /* XXX */
 	},
 	{
 		.value = SIGALG_GOSTR01_GOST94,
 		.md = EVP_gostr341194,
 		.key_type = EVP_PKEY_GOSTR01,
-		.pkey_idx = SSL_PKEY_GOST01,
 	},
 #endif
 	{
 		.value = SIGALG_RSA_PSS_RSAE_SHA256,
 		.md = EVP_sha256,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.flags = SIGALG_FLAG_RSA_PSS,
 	},
 	{
 		.value = SIGALG_RSA_PSS_RSAE_SHA384,
 		.md = EVP_sha384,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.flags = SIGALG_FLAG_RSA_PSS,
 	},
 	{
 		.value = SIGALG_RSA_PSS_RSAE_SHA512,
 		.md = EVP_sha512,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.flags = SIGALG_FLAG_RSA_PSS,
 	},
 	{
 		.value = SIGALG_RSA_PSS_PSS_SHA256,
 		.md = EVP_sha256,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.flags = SIGALG_FLAG_RSA_PSS,
 	},
 	{
 		.value = SIGALG_RSA_PSS_PSS_SHA384,
 		.md = EVP_sha384,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.flags = SIGALG_FLAG_RSA_PSS,
 	},
 	{
 		.value = SIGALG_RSA_PSS_PSS_SHA512,
 		.md = EVP_sha512,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.flags = SIGALG_FLAG_RSA_PSS,
 	},
 	{
 		.value = SIGALG_RSA_PKCS1_SHA224,
 		.md = EVP_sha224,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 	},
 	{
 		.value = SIGALG_ECDSA_SECP224R1_SHA224,
 		.md = EVP_sha224,
 		.key_type = EVP_PKEY_EC,
-		.pkey_idx = SSL_PKEY_ECC,
 	},
 	{
 		.value = SIGALG_RSA_PKCS1_SHA1,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.md = EVP_sha1,
 	},
 	{
 		.value = SIGALG_ECDSA_SHA1,
 		.key_type = EVP_PKEY_EC,
 		.md = EVP_sha1,
-		.pkey_idx = SSL_PKEY_ECC,
 	},
 	{
 		.value = SIGALG_RSA_PKCS1_MD5_SHA1,
 		.key_type = EVP_PKEY_RSA,
-		.pkey_idx = SSL_PKEY_RSA_SIGN,
 		.md = EVP_md5_sha1,
 	},
 	{
@@ -163,19 +143,31 @@ const struct ssl_sigalg sigalgs[] = {
 	},
 };
 
-/* Sigalgs for tls 1.2, in preference order, */
-uint16_t tls12_sigalgs[] = {
+/* Sigalgs for tls 1.3, in preference order, */
+uint16_t tls13_sigalgs[] = {
+	SIGALG_RSA_PSS_RSAE_SHA512,
 	SIGALG_RSA_PKCS1_SHA512,
-	SIGALG_ECDSA_SECP512R1_SHA512,
-	SIGALG_GOSTR12_512_STREEBOG_512,
+	SIGALG_ECDSA_SECP521R1_SHA512,
+	SIGALG_RSA_PSS_RSAE_SHA384,
 	SIGALG_RSA_PKCS1_SHA384,
 	SIGALG_ECDSA_SECP384R1_SHA384,
+	SIGALG_RSA_PSS_RSAE_SHA256,
+	SIGALG_RSA_PKCS1_SHA256,
+	SIGALG_ECDSA_SECP256R1_SHA256,
+};
+size_t tls13_sigalgs_len = (sizeof(tls13_sigalgs) / sizeof(tls13_sigalgs[0]));
+
+/* Sigalgs for tls 1.2, in preference order, */
+uint16_t tls12_sigalgs[] = {
+	SIGALG_RSA_PSS_RSAE_SHA512,
+	SIGALG_RSA_PKCS1_SHA512,
+	SIGALG_ECDSA_SECP521R1_SHA512,
+	SIGALG_RSA_PSS_RSAE_SHA384,
+	SIGALG_RSA_PKCS1_SHA384,
+	SIGALG_ECDSA_SECP384R1_SHA384,
+	SIGALG_RSA_PSS_RSAE_SHA256,
 	SIGALG_RSA_PKCS1_SHA256,
 	SIGALG_ECDSA_SECP256R1_SHA256,
-	SIGALG_GOSTR12_256_STREEBOG_256,
-	SIGALG_GOSTR01_GOST94,
-	SIGALG_RSA_PKCS1_SHA224,
-	SIGALG_ECDSA_SECP224R1_SHA224,
 	SIGALG_RSA_PKCS1_SHA1, /* XXX */
 	SIGALG_ECDSA_SHA1,     /* XXX */
 };
@@ -197,16 +189,11 @@ ssl_sigalg_lookup(uint16_t sigalg)
 const struct ssl_sigalg *
 ssl_sigalg(uint16_t sigalg, uint16_t *values, size_t len)
 {
-	const struct ssl_sigalg *sap;
 	int i;
 
 	for (i = 0; i < len; i++) {
 		if (values[i] == sigalg)
-			break;
-	}
-	if (values[i] == sigalg) {
-		if ((sap = ssl_sigalg_lookup(sigalg)) != NULL)
-			return sap;
+			return ssl_sigalg_lookup(sigalg);
 	}
 
 	return NULL;
@@ -239,7 +226,8 @@ ssl_sigalgs_build(CBB *cbb, uint16_t *values, size_t len)
 }
 
 int
-ssl_sigalg_pkey_ok(const struct ssl_sigalg *sigalg, EVP_PKEY *pkey)
+ssl_sigalg_pkey_ok(const struct ssl_sigalg *sigalg, EVP_PKEY *pkey,
+    int check_curve)
 {
 	if (sigalg == NULL || pkey == NULL)
 		return 0;
@@ -256,15 +244,88 @@ ssl_sigalg_pkey_ok(const struct ssl_sigalg *sigalg, EVP_PKEY *pkey)
 			return 0;
 	}
 
-	if (pkey->type == EVP_PKEY_EC) {
+	if (pkey->type == EVP_PKEY_EC && check_curve) {
+		/* Curve must match for EC keys. */
 		if (sigalg->curve_nid == 0)
 			return 0;
-		/* Curve must match for EC keys */
 		if (EC_GROUP_get_curve_name(EC_KEY_get0_group
 		    (EVP_PKEY_get0_EC_KEY(pkey))) != sigalg->curve_nid) {
-			return 1; /* XXX www.videolan.org curve mismatch */
+			return 0;
 		}
 	}
 
 	return 1;
 }
+
+const struct ssl_sigalg *
+ssl_sigalg_select(SSL *s, EVP_PKEY *pkey)
+{
+	uint16_t *tls_sigalgs = tls12_sigalgs;
+	size_t tls_sigalgs_len = tls12_sigalgs_len;
+	int check_curve = 0;
+	CBS cbs;
+
+	if (TLS1_get_version(s) >= TLS1_3_VERSION) {
+		tls_sigalgs = tls13_sigalgs;
+		tls_sigalgs_len = tls13_sigalgs_len;
+		check_curve = 1;
+	}
+
+	/* Pre TLS 1.2 defaults */
+	if (!SSL_USE_SIGALGS(s)) {
+		switch (pkey->type) {
+		case EVP_PKEY_RSA:
+			return ssl_sigalg_lookup(SIGALG_RSA_PKCS1_MD5_SHA1);
+		case EVP_PKEY_EC:
+			return ssl_sigalg_lookup(SIGALG_ECDSA_SHA1);
+#ifndef OPENSSL_NO_GOST
+		case EVP_PKEY_GOSTR01:
+			return ssl_sigalg_lookup(SIGALG_GOSTR01_GOST94);
+#endif
+		}
+		SSLerror(s, SSL_R_UNKNOWN_PKEY_TYPE);
+		return (NULL);
+	}
+
+	/*
+	 * RFC 5246 allows a TLS 1.2 client to send no sigalgs, in
+	 * which case the server must use the the default.
+	 */
+	if (TLS1_get_version(s) < TLS1_3_VERSION &&
+	    S3I(s)->hs.sigalgs == NULL) {
+		switch (pkey->type) {
+		case EVP_PKEY_RSA:
+			return ssl_sigalg_lookup(SIGALG_RSA_PKCS1_SHA1);
+		case EVP_PKEY_EC:
+			return ssl_sigalg_lookup(SIGALG_ECDSA_SHA1);
+#ifndef OPENSSL_NO_GOST
+		case EVP_PKEY_GOSTR01:
+			return ssl_sigalg_lookup(SIGALG_GOSTR01_GOST94);
+#endif
+		}
+		SSLerror(s, SSL_R_UNKNOWN_PKEY_TYPE);
+		return (NULL);
+	}
+
+	/*
+	 * If we get here, we have client or server sent sigalgs, use one.
+	 */
+	CBS_init(&cbs, S3I(s)->hs.sigalgs, S3I(s)->hs.sigalgs_len);
+	while (CBS_len(&cbs) > 0) {
+		uint16_t sig_alg;
+		const struct ssl_sigalg *sigalg;
+
+		if (!CBS_get_u16(&cbs, &sig_alg))
+			return 0;
+
+		if ((sigalg = ssl_sigalg(sig_alg, tls_sigalgs,
+		    tls_sigalgs_len)) == NULL)
+			continue;
+
+		if (ssl_sigalg_pkey_ok(sigalg, pkey, check_curve))
+			return sigalg;
+	}
+
+	SSLerror(s, SSL_R_UNKNOWN_PKEY_TYPE);
+	return NULL;
+}
