@@ -1,4 +1,4 @@
-/* $OpenBSD: tls13_record_layer.c,v 1.9 2019/03/17 15:13:23 jsing Exp $ */
+/* $OpenBSD: tls13_record_layer.c,v 1.30 2020/03/16 15:11:35 tb Exp $ */
 /*
  * Copyright (c) 2018, 2019 Joel Sing <jsing@openbsd.org>
  *
@@ -17,14 +17,22 @@
 
 #include "ssl_locl.h"
 
-#include <openssl/curve25519.h>
-
 #include "tls13_internal.h"
 #include "tls13_record.h"
 
+static ssize_t tls13_record_layer_write_chunk(struct tls13_record_layer *rl,
+    uint8_t content_type, const uint8_t *buf, size_t n);
+static ssize_t tls13_record_layer_write_record(struct tls13_record_layer *rl,
+    uint8_t content_type, const uint8_t *content, size_t content_len);
+
 struct tls13_record_layer {
-	int change_cipher_spec_seen;
+	uint16_t legacy_version;
+
+	int ccs_allowed;
+	int ccs_seen;
 	int handshake_completed;
+	int legacy_alerts_allowed;
+	int phh;
 
 	/*
 	 * Read and/or write channels are closed due to an alert being
@@ -36,7 +44,22 @@ struct tls13_record_layer {
 	int write_closed;
 
 	struct tls13_record *rrec;
+
 	struct tls13_record *wrec;
+	uint8_t wrec_content_type;
+	size_t wrec_appdata_len;
+	size_t wrec_content_len;
+
+	/* Pending alert messages. */
+	uint8_t *alert_data;
+	size_t alert_len;
+	uint8_t alert_level;
+	uint8_t alert_desc;
+
+	/* Pending post-handshake handshake messages (RFC 8446, section 4.6). */
+	CBS phh_cbs;
+	uint8_t *phh_data;
+	size_t phh_len;
 
 	/* Buffer containing plaintext from opened records. */
 	uint8_t rbuf_content_type;
@@ -58,7 +81,8 @@ struct tls13_record_layer {
 
 	/* Record callbacks. */
 	tls13_alert_cb alert_cb;
-	tls13_post_handshake_cb post_handshake_cb;
+	tls13_phh_recv_cb phh_recv_cb;
+	tls13_phh_sent_cb phh_sent_cb;
 
 	/* Wire read/write callbacks. */
 	tls13_read_cb wire_read;
@@ -92,7 +116,9 @@ tls13_record_layer_wrec_free(struct tls13_record_layer *rl)
 
 struct tls13_record_layer *
 tls13_record_layer_new(tls13_read_cb wire_read, tls13_write_cb wire_write,
-    tls13_alert_cb alert_cb, tls13_post_handshake_cb post_handshake_cb,
+    tls13_alert_cb alert_cb,
+    tls13_phh_recv_cb phh_recv_cb,
+    tls13_phh_sent_cb phh_sent_cb,
     void *cb_arg)
 {
 	struct tls13_record_layer *rl;
@@ -100,10 +126,13 @@ tls13_record_layer_new(tls13_read_cb wire_read, tls13_write_cb wire_write,
 	if ((rl = calloc(1, sizeof(struct tls13_record_layer))) == NULL)
 		return NULL;
 
+	rl->legacy_version = TLS1_2_VERSION;
+
 	rl->wire_read = wire_read;
 	rl->wire_write = wire_write;
 	rl->alert_cb = alert_cb;
-	rl->post_handshake_cb = post_handshake_cb;
+	rl->phh_recv_cb = phh_recv_cb;
+	rl->phh_sent_cb = phh_sent_cb;
 	rl->cb_arg = cb_arg;
 
 	return rl;
@@ -131,18 +160,31 @@ tls13_record_layer_free(struct tls13_record_layer *rl)
 	freezero(rl, sizeof(struct tls13_record_layer));
 }
 
-static int
+void
+tls13_record_layer_rbuf(struct tls13_record_layer *rl, CBS *cbs)
+{
+	CBS_dup(&rl->rbuf_cbs, cbs);
+}
+
+uint8_t tls13_max_seq_num[TLS13_RECORD_SEQ_NUM_LEN] = {
+	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
+};
+
+int
 tls13_record_layer_inc_seq_num(uint8_t *seq_num)
 {
-	size_t i;
+	int i;
 
-	for (i = TLS13_RECORD_SEQ_NUM_LEN - 1; i > 0; i--) {
+	/* RFC 8446 section 5.3 - sequence numbers must not wrap. */
+	if (memcmp(seq_num, tls13_max_seq_num, TLS13_RECORD_SEQ_NUM_LEN) == 0)
+		return 0;
+
+	for (i = TLS13_RECORD_SEQ_NUM_LEN - 1; i >= 0; i--) {
 		if (++seq_num[i] != 0)
 			break;
 	}
 
-	/* RFC 8446 section 5.3 - sequence numbers must not wrap. */
-	return (i != 0 || seq_num[0] != 0);
+	return 1;
 }
 
 static int
@@ -165,6 +207,18 @@ tls13_record_layer_update_nonce(struct tls13_secret *nonce,
 	return 1;
 }
 
+void
+tls13_record_layer_allow_ccs(struct tls13_record_layer *rl, int allow)
+{
+	rl->ccs_allowed = allow;
+}
+
+void
+tls13_record_layer_allow_legacy_alerts(struct tls13_record_layer *rl, int allow)
+{
+	rl->legacy_alerts_allowed = allow;
+}
+
 void
 tls13_record_layer_set_aead(struct tls13_record_layer *rl,
     const EVP_AEAD *aead)
@@ -179,6 +233,13 @@ tls13_record_layer_set_hash(struct tls13_record_layer *rl,
 	rl->hash = hash;
 }
 
+void
+tls13_record_layer_set_legacy_version(struct tls13_record_layer *rl,
+    uint16_t version)
+{
+	rl->legacy_version = version;
+}
+
 void
 tls13_record_layer_handshake_completed(struct tls13_record_layer *rl)
 {
@@ -200,17 +261,19 @@ tls13_record_layer_process_alert(struct tls13_record_layer *rl)
 	 * read channel closure (close_notify) or termination (all others).
 	 */
 	if (rl->rbuf == NULL)
-		goto err;
+		return TLS13_IO_FAILURE;
+
 	if (rl->rbuf_content_type != SSL3_RT_ALERT)
-		goto err;
+		return TLS13_IO_FAILURE;
 
 	if (!CBS_get_u8(&rl->rbuf_cbs, &alert_level))
-		goto err; /* XXX - decode error alert. */
+		return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+
 	if (!CBS_get_u8(&rl->rbuf_cbs, &alert_desc))
-		goto err; /* XXX - decode error alert. */
+		return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
 
 	if (CBS_len(&rl->rbuf_cbs) != 0)
-		goto err; /* XXX - decode error alert. */
+		return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
 
 	tls13_record_layer_rbuf_free(rl);
 
@@ -224,28 +287,143 @@ tls13_record_layer_process_alert(struct tls13_record_layer *rl)
 		ret = TLS13_IO_EOF;
 	} else if (alert_desc == SSL_AD_USER_CANCELLED) {
 		/* Ignored at the record layer. */
-		ret = TLS13_IO_WANT_POLLIN;
+		ret = TLS13_IO_WANT_RETRY;
 	} else if (alert_level == SSL3_AL_FATAL) {
 		rl->read_closed = 1;
 		rl->write_closed = 1;
-		ret = TLS13_IO_EOF;
+		ret = TLS13_IO_ALERT;
+	} else if (rl->legacy_alerts_allowed && alert_level == SSL3_AL_WARNING) {
+		/* Ignored and not passed to the callback. */
+		return TLS13_IO_WANT_RETRY;
 	} else {
-		/* XXX - decode error alert. */
-		return TLS13_IO_FAILURE;
+		return tls13_send_alert(rl, SSL_AD_ILLEGAL_PARAMETER);
 	}
 
 	rl->alert_cb(alert_desc, rl->cb_arg);
 
- err:
 	return ret;
 }
 
-int
-tls13_record_layer_send_alert(struct tls13_record_layer *rl,
+static ssize_t
+tls13_record_layer_send_alert(struct tls13_record_layer *rl)
+{
+	ssize_t ret;
+
+	/* This has to fit into a single record, per RFC 8446 section 5.1. */
+	if ((ret = tls13_record_layer_write_record(rl, SSL3_RT_ALERT,
+	    rl->alert_data, rl->alert_len)) != rl->alert_len) {
+		if (ret == TLS13_IO_EOF)
+			ret = TLS13_IO_ALERT;
+		return ret;
+	}
+
+	freezero(rl->alert_data, rl->alert_len);
+	rl->alert_data = NULL;
+	rl->alert_len = 0;
+
+	if (rl->alert_desc == SSL_AD_CLOSE_NOTIFY) {
+		rl->write_closed = 1;
+		ret = TLS13_IO_SUCCESS;
+	} else if (rl->alert_desc == SSL_AD_USER_CANCELLED) {
+		/* Ignored at the record layer. */
+		ret = TLS13_IO_SUCCESS;
+	} else {
+		rl->read_closed = 1;
+		rl->write_closed = 1;
+		ret = TLS13_IO_ALERT;
+	}
+
+	return ret;
+}
+
+static ssize_t
+tls13_record_layer_send_phh(struct tls13_record_layer *rl)
+{
+	ssize_t ret;
+
+	/* Push out pending post-handshake handshake messages. */
+	if ((ret = tls13_record_layer_write_chunk(rl, SSL3_RT_HANDSHAKE,
+	    CBS_data(&rl->phh_cbs), CBS_len(&rl->phh_cbs))) < 0)
+		return ret;
+	if (!CBS_skip(&rl->phh_cbs, ret))
+		return TLS13_IO_FAILURE;
+	if (CBS_len(&rl->phh_cbs) != 0)
+		return TLS13_IO_WANT_RETRY;
+
+	freezero(rl->phh_data, rl->phh_len);
+	rl->phh_data = NULL;
+	rl->phh_len = 0;
+
+	CBS_init(&rl->phh_cbs, rl->phh_data, rl->phh_len);
+
+	rl->phh_sent_cb(rl->cb_arg);
+
+	return TLS13_IO_SUCCESS;
+}
+
+ssize_t
+tls13_record_layer_send_pending(struct tls13_record_layer *rl)
+{
+	/*
+	 * If an alert is pending, then it needs to be sent. However,
+	 * if we're already part of the way through sending post-handshake
+	 * handshake messages, then we need to finish that first...
+	 */
+
+	if (rl->phh_data != NULL && CBS_len(&rl->phh_cbs) != rl->phh_len)
+		return tls13_record_layer_send_phh(rl);
+
+	if (rl->alert_data != NULL)
+		return tls13_record_layer_send_alert(rl);
+
+	if (rl->phh_data != NULL)
+		return tls13_record_layer_send_phh(rl);
+
+	return TLS13_IO_SUCCESS;
+}
+
+static ssize_t
+tls13_record_layer_alert(struct tls13_record_layer *rl,
     uint8_t alert_level, uint8_t alert_desc)
 {
-	/* XXX - implement. */
-	return -1;
+	CBB cbb;
+
+	if (rl->alert_data != NULL)
+		return TLS13_IO_FAILURE;
+
+	if (!CBB_init(&cbb, 0))
+		goto err;
+
+	if (!CBB_add_u8(&cbb, alert_level))
+		goto err;
+	if (!CBB_add_u8(&cbb, alert_desc))
+		goto err;
+	if (!CBB_finish(&cbb, &rl->alert_data, &rl->alert_len))
+		goto err;
+
+	rl->alert_level = alert_level;
+	rl->alert_desc = alert_desc;
+
+	return tls13_record_layer_send_pending(rl);
+
+ err:
+	CBB_cleanup(&cbb);
+
+	return TLS13_IO_FAILURE;
+}
+
+ssize_t
+tls13_record_layer_phh(struct tls13_record_layer *rl, CBS *cbs)
+{
+	if (rl->phh_data != NULL)
+		return TLS13_IO_FAILURE;
+
+	if (!CBS_stow(cbs, &rl->phh_data, &rl->phh_len))
+		return TLS13_IO_FAILURE;
+
+	CBS_init(&rl->phh_cbs, rl->phh_data, rl->phh_len);
+
+	return tls13_record_layer_send_pending(rl);
 }
 
 static int
@@ -408,6 +586,9 @@ tls13_record_layer_open_record_protected(struct tls13_record_layer *rl)
 static int
 tls13_record_layer_open_record(struct tls13_record_layer *rl)
 {
+	if (rl->handshake_completed && rl->aead == NULL)
+		return 0;
+
 	if (rl->aead == NULL)
 		return tls13_record_layer_open_record_plaintext(rl);
 
@@ -420,15 +601,11 @@ tls13_record_layer_seal_record_plaintext(struct tls13_record_layer *rl,
 {
 	uint8_t *data = NULL;
 	size_t data_len = 0;
-	uint16_t version;
 	CBB cbb, body;
 
 	if (rl->aead != NULL)
 		return 0;
 
-	/* XXX - TLS1_VERSION for first client hello... */
-	version = TLS1_2_VERSION;
-
 	/*
 	 * We're still operating in plaintext mode, so just copy the
 	 * content into the record.
@@ -438,7 +615,7 @@ tls13_record_layer_seal_record_plaintext(struct tls13_record_layer *rl,
 
 	if (!CBB_add_u8(&cbb, content_type))
 		goto err;
-	if (!CBB_add_u16(&cbb, version))
+	if (!CBB_add_u16(&cbb, rl->legacy_version))
 		goto err;
 	if (!CBB_add_u16_length_prefixed(&cbb, &body))
 		goto err;
@@ -451,6 +628,9 @@ tls13_record_layer_seal_record_plaintext(struct tls13_record_layer *rl,
 	if (!tls13_record_set_data(rl->wrec, data, data_len))
 		goto err;
 
+	rl->wrec_content_len = content_len;
+	rl->wrec_content_type = content_type;
+
 	return 1;
 
  err:
@@ -542,6 +722,9 @@ tls13_record_layer_seal_record_protected(struct tls13_record_layer *rl,
 	if (!tls13_record_set_data(rl->wrec, data, data_len))
 		goto err;
 
+	rl->wrec_content_len = content_len;
+	rl->wrec_content_type = content_type;
+
 	data = NULL;
 	data_len = 0;
 
@@ -561,6 +744,9 @@ static int
 tls13_record_layer_seal_record(struct tls13_record_layer *rl,
     uint8_t content_type, const uint8_t *content, size_t content_len)
 {
+	if (rl->handshake_completed && rl->aead == NULL)
+		return 0;
+
 	tls13_record_layer_wrec_free(rl);
 
 	if ((rl->wrec = tls13_record_new()) == NULL)
@@ -601,26 +787,17 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 	 * ignored.
 	 */
 	if (content_type == SSL3_RT_CHANGE_CIPHER_SPEC) {
-		/* XXX - need to check after ClientHello, before Finished. */
-		if (rl->handshake_completed || rl->change_cipher_spec_seen) {
-			/* XXX - unexpected message alert. */
-			goto err;
-		}
-		if (!tls13_record_content(rl->rrec, &cbs)) {
-			/* XXX - decode error alert. */
-			goto err;
-		}
-		if (!CBS_get_u8(&cbs, &ccs)) {
-			/* XXX - decode error alert. */
-			goto err;
-		}
-		if (ccs != 1) {
-			/* XXX - something alert. */
-			goto err;
-		}
-		rl->change_cipher_spec_seen = 1;
+		if (!rl->ccs_allowed || rl->ccs_seen)
+			return tls13_send_alert(rl, SSL_AD_UNEXPECTED_MESSAGE);
+		if (!tls13_record_content(rl->rrec, &cbs))
+			return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+		if (!CBS_get_u8(&cbs, &ccs))
+			return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+		if (ccs != 1)
+			return tls13_send_alert(rl, SSL_AD_ILLEGAL_PARAMETER);
+		rl->ccs_seen = 1;
 		tls13_record_layer_rrec_free(rl);
-		return TLS13_IO_WANT_POLLIN;
+		return TLS13_IO_WANT_RETRY;
 	}
 
 	/*
@@ -628,10 +805,8 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 	 * protected application data messages (aside from the
 	 * dummy ChangeCipherSpec messages, handled above).
 	 */
-	if (rl->aead != NULL && content_type != SSL3_RT_APPLICATION_DATA) {
-		/* XXX - unexpected message alert. */
-		goto err;
-	}
+	if (rl->aead != NULL && content_type != SSL3_RT_APPLICATION_DATA)
+		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
 
 	if (!tls13_record_layer_open_record(rl))
 		goto err;
@@ -646,15 +821,12 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 		break;
 
 	case SSL3_RT_APPLICATION_DATA:
-		if (!rl->handshake_completed) {
-			/* XXX - unexpected message alert. */
-			goto err;
-		}
+		if (!rl->handshake_completed)
+			return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
 		break;
 
 	default:
-		/* XXX - unexpected message alert. */
-		goto err;
+		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
 	}
 
 	return TLS13_IO_SUCCESS;
@@ -664,17 +836,26 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 }
 
 ssize_t
-tls13_record_layer_read(struct tls13_record_layer *rl, uint8_t content_type,
-    uint8_t *buf, size_t n)
+tls13_record_layer_pending(struct tls13_record_layer *rl, uint8_t content_type)
+{
+	if (rl->rbuf_content_type != content_type)
+		return 0;
+
+	return CBS_len(&rl->rbuf_cbs);
+}
+
+static ssize_t
+tls13_record_layer_read_internal(struct tls13_record_layer *rl,
+    uint8_t content_type, uint8_t *buf, size_t n, int peek)
 {
 	ssize_t ret;
 
+	if ((ret = tls13_record_layer_send_pending(rl)) != TLS13_IO_SUCCESS)
+		return ret;
+
 	if (rl->read_closed)
 		return TLS13_IO_EOF;
 
-	/* XXX - loop here with record and byte limits. */
-	/* XXX - send alert... */
-
 	/* If necessary, pull up the next record. */
 	if (CBS_len(&rl->rbuf_cbs) == 0) {
 		if ((ret = tls13_record_layer_read_record(rl)) <= 0)
@@ -682,6 +863,14 @@ tls13_record_layer_read(struct tls13_record_layer *rl, uint8_t content_type,
 
 		/* XXX - need to check record version. */
 	}
+
+	/*
+	 * If we are in post handshake handshake mode, we may not see
+	 * any record type that isn't a handshake until we are done.
+	 */
+	if (rl->phh && rl->rbuf_content_type != SSL3_RT_HANDSHAKE)
+		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
+
 	if (rl->rbuf_content_type != content_type) {
 		/*
 		 * Handshake content can appear as post-handshake messages (yup,
@@ -691,14 +880,50 @@ tls13_record_layer_read(struct tls13_record_layer *rl, uint8_t content_type,
 		 */
 		if (rl->rbuf_content_type == SSL3_RT_HANDSHAKE) {
 			if (rl->handshake_completed) {
-				/* XXX - call callback, drop for now... */
+				rl->phh = 1;
+				ret = TLS13_IO_FAILURE;
+
+				/*
+				 * The post handshake handshake
+				 * receive callback is allowed to
+				 * return:
+				 *
+				 * TLS13_IO_WANT_POLLIN ->
+				 * I need more handshake data.
+				 *
+				 * TLS13_IO_WANT_POLLOUT -> I got the
+				 * whole handshake message, and have
+				 * enqueued a response
+				 *
+				 * TLS13_IO_SUCCESS -> I got the whole handshake,
+				 * nothing more to do
+				 *
+				 * TLS13_IO_FAILURE -> something broke.
+				 */
+				if (rl->phh_recv_cb != NULL) {
+					ret = rl->phh_recv_cb(
+					    rl->cb_arg, &rl->rbuf_cbs);
+				}
+
 				tls13_record_layer_rbuf_free(rl);
-				return TLS13_IO_WANT_POLLIN;
+
+				if (ret == TLS13_IO_WANT_POLLIN)
+					return ret;
+
+				/*
+				 * leave post handshake handshake mode
+				 * if we do not need more handshake data
+				 */
+				rl->phh = 0;
+
+				if (ret == TLS13_IO_SUCCESS)
+					return TLS13_IO_WANT_RETRY;
+
+				return ret;
 			}
 		}
 
-		/* XXX - unexpected message alert. */
-		goto err;
+		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
 	}
 
 	if (n > CBS_len(&rl->rbuf_cbs))
@@ -706,8 +931,11 @@ tls13_record_layer_read(struct tls13_record_layer *rl, uint8_t content_type,
 
 	/* XXX - CBS_memcpy? CBS_copy_bytes? */
 	memcpy(buf, CBS_data(&rl->rbuf_cbs), n);
-	if (!CBS_skip(&rl->rbuf_cbs, n))
-		goto err;
+
+	if (!peek) {
+		if (!CBS_skip(&rl->rbuf_cbs, n))
+			goto err;
+	}
 
 	if (CBS_len(&rl->rbuf_cbs) == 0)
 		tls13_record_layer_rbuf_free(rl);
@@ -718,6 +946,32 @@ tls13_record_layer_read(struct tls13_record_layer *rl, uint8_t content_type,
 	return TLS13_IO_FAILURE;
 }
 
+ssize_t
+tls13_record_layer_peek(struct tls13_record_layer *rl, uint8_t content_type,
+    uint8_t *buf, size_t n)
+{
+	ssize_t ret;
+
+	do {
+		ret = tls13_record_layer_read_internal(rl, content_type, buf, n, 1);
+	} while (ret == TLS13_IO_WANT_RETRY);
+
+	return ret;
+}
+
+ssize_t
+tls13_record_layer_read(struct tls13_record_layer *rl, uint8_t content_type,
+    uint8_t *buf, size_t n)
+{
+	ssize_t ret;
+
+	do {
+		ret = tls13_record_layer_read_internal(rl, content_type, buf, n, 0);
+	} while (ret == TLS13_IO_WANT_RETRY);
+
+	return ret;
+}
+
 static ssize_t
 tls13_record_layer_write_record(struct tls13_record_layer *rl,
     uint8_t content_type, const uint8_t *content, size_t content_len)
@@ -727,16 +981,38 @@ tls13_record_layer_write_record(struct tls13_record_layer *rl,
 	if (rl->write_closed)
 		return TLS13_IO_EOF;
 
+	/*
+	 * If we pushed out application data while handling other messages,
+	 * we need to return content length on the next call.
+	 */
+	if (content_type == SSL3_RT_APPLICATION_DATA &&
+	    rl->wrec_appdata_len != 0) {
+		ret = rl->wrec_appdata_len;
+		rl->wrec_appdata_len = 0;
+		return ret;
+	}
+
 	/* See if there is an existing record and attempt to push it out... */
 	if (rl->wrec != NULL) {
 		if ((ret = tls13_record_send(rl->wrec, rl->wire_write,
 		    rl->cb_arg)) <= 0)
 			return ret;
-
 		tls13_record_layer_wrec_free(rl);
 
-		/* XXX - could be pushing out different data... */
-		return content_len;
+		if (rl->wrec_content_type == content_type) {
+			ret = rl->wrec_content_len;
+			rl->wrec_content_len = 0;
+			rl->wrec_content_type = 0;
+			return ret;
+		}
+
+		/*
+		 * The only partial record type should be application data.
+		 * All other cases are handled to completion.
+		 */
+		if (rl->wrec_content_type != SSL3_RT_APPLICATION_DATA)
+			return TLS13_IO_FAILURE;
+		rl->wrec_appdata_len = rl->wrec_content_len;
 	}
 
 	if (content_len > TLS13_RECORD_MAX_PLAINTEXT_LEN)
@@ -757,8 +1033,8 @@ tls13_record_layer_write_record(struct tls13_record_layer *rl,
 }
 
 static ssize_t
-tls13_record_layer_write(struct tls13_record_layer *rl, uint8_t content_type,
-    const uint8_t *buf, size_t n)
+tls13_record_layer_write_chunk(struct tls13_record_layer *rl,
+    uint8_t content_type, const uint8_t *buf, size_t n)
 {
 	if (n > TLS13_RECORD_MAX_PLAINTEXT_LEN)
 		n = TLS13_RECORD_MAX_PLAINTEXT_LEN;
@@ -766,6 +1042,25 @@ tls13_record_layer_write(struct tls13_record_layer *rl, uint8_t content_type,
 	return tls13_record_layer_write_record(rl, content_type, buf, n);
 }
 
+static ssize_t
+tls13_record_layer_write(struct tls13_record_layer *rl, uint8_t content_type,
+    const uint8_t *buf, size_t n)
+{
+	ssize_t ret;
+
+	do {
+		ret = tls13_record_layer_send_pending(rl);
+	} while (ret == TLS13_IO_WANT_RETRY);
+	if (ret != TLS13_IO_SUCCESS)
+		return ret;
+
+	do {
+		ret = tls13_record_layer_write_chunk(rl, content_type, buf, n);
+	} while (ret == TLS13_IO_WANT_RETRY);
+
+	return ret;
+}
+
 ssize_t
 tls13_read_handshake_data(struct tls13_record_layer *rl, uint8_t *buf, size_t n)
 {
@@ -779,6 +1074,24 @@ tls13_write_handshake_data(struct tls13_record_layer *rl, const uint8_t *buf,
 	return tls13_record_layer_write(rl, SSL3_RT_HANDSHAKE, buf, n);
 }
 
+ssize_t
+tls13_pending_application_data(struct tls13_record_layer *rl)
+{
+	if (!rl->handshake_completed)
+		return 0;
+
+	return tls13_record_layer_pending(rl, SSL3_RT_APPLICATION_DATA);
+}
+
+ssize_t
+tls13_peek_application_data(struct tls13_record_layer *rl, uint8_t *buf, size_t n)
+{
+	if (!rl->handshake_completed)
+		return TLS13_IO_FAILURE;
+
+	return tls13_record_layer_peek(rl, SSL3_RT_APPLICATION_DATA, buf, n);
+}
+
 ssize_t
 tls13_read_application_data(struct tls13_record_layer *rl, uint8_t *buf, size_t n)
 {
@@ -797,3 +1110,20 @@ tls13_write_application_data(struct tls13_record_layer *rl, const uint8_t *buf,
 
 	return tls13_record_layer_write(rl, SSL3_RT_APPLICATION_DATA, buf, n);
 }
+
+ssize_t
+tls13_send_alert(struct tls13_record_layer *rl, uint8_t alert_desc)
+{
+	uint8_t alert_level = SSL3_AL_FATAL;
+	ssize_t ret;
+
+	if (alert_desc == SSL_AD_CLOSE_NOTIFY ||
+	    alert_desc == SSL_AD_USER_CANCELLED)
+		alert_level = SSL3_AL_WARNING;
+
+	do {
+		ret = tls13_record_layer_alert(rl, alert_level, alert_desc);
+	} while (ret == TLS13_IO_WANT_RETRY);
+
+	return ret;
+}
